from dagster.api.snapshot_partition import sync_get_external_partition
from dagster.core.host_representation import ExternalPartitionData

from .utils import get_bar_repo_handle


def test_external_partitions_api():
    repository_handle = get_bar_repo_handle()
    partition_data = sync_get_external_partition(repository_handle, 'baz_partitions', 'c')
    assert isinstance(partition_data, ExternalPartitionData)
    assert partition_data.run_config
    assert partition_data.run_config['solids']['do_input']['inputs']['x']['value'] == 'c'



from dagster.seven import mock
from dagster.utils.backcompat import rename_warning


def test_no_additional_warn_text():
    with mock.patch('warnings.warn') as warn_mock:
        rename_warning('a_new_name', 'an_old_name', '0.3.0')
        warn_mock.assert_called_once_with(
            '"an_old_name" is deprecated and will be removed in 0.3.0, use "a_new_name" instead.',
            stacklevel=3,
        )

    with mock.patch('warnings.warn') as warn_mock:
        rename_warning('a_new_name', 'an_old_name', '0.3.0', 'Additional compelling text.')
        warn_mock.assert_called_once_with(
            (
                '"an_old_name" is deprecated and will be removed in 0.3.0, use "a_new_name" '
                'instead. Additional compelling text.'
            ),
            stacklevel=3,
        )



import datetime

import pytest

from dagster import daily_schedule, lambda_solid, pipeline, repository, seven
from dagster.api.launch_scheduled_execution import sync_launch_scheduled_execution
from dagster.core.definitions.reconstructable import ReconstructableRepository
from dagster.core.errors import DagsterSubprocessError
from dagster.core.instance import DagsterInstance
from dagster.core.scheduler import (
    ScheduleTickStatus,
    ScheduledExecutionFailed,
    ScheduledExecutionSkipped,
    ScheduledExecutionSuccess,
)
from dagster.core.test_utils import environ


def _throw(_context):
    raise Exception('bananas')


def _never(_context):
    return False


@lambda_solid
def the_solid():
    return '0.8.0 was a lot of work'


@pipeline
def the_pipeline():
    the_solid()


@daily_schedule(
    pipeline_name='the_pipeline', start_date=datetime.datetime.now() - datetime.timedelta(days=1),
)
def simple_schedule(_context):
    return {}


@daily_schedule(
    pipeline_name='the_pipeline', start_date=datetime.datetime.now() - datetime.timedelta(days=1),
)
def bad_env_fn_schedule():  # forgot context arg
    return {}


@daily_schedule(
    pipeline_name='the_pipeline',
    start_date=datetime.datetime.now() - datetime.timedelta(days=1),
    should_execute=_throw,
)
def bad_should_execute_schedule(_context):
    return {}


@daily_schedule(
    pipeline_name='the_pipeline',
    start_date=datetime.datetime.now() - datetime.timedelta(days=1),
    should_execute=_never,
)
def skip_schedule(_context):
    return {}


@repository
def the_repo():
    return [
        the_pipeline,
        simple_schedule,
        bad_env_fn_schedule,
        bad_should_execute_schedule,
        skip_schedule,
    ]


def test_launch_scheduled_execution():
    with seven.TemporaryDirectory() as temp_dir:
        with environ({'DAGSTER_HOME': temp_dir}):
            instance = DagsterInstance.get()

            recon_repo = ReconstructableRepository.for_file(__file__, 'the_repo')
            simple = recon_repo.get_reconstructable_schedule('simple_schedule')
            result = sync_launch_scheduled_execution(simple.get_origin())

            assert isinstance(result, ScheduledExecutionSuccess)

            run = instance.get_run_by_id(result.run_id)
            assert run.is_success

            ticks = instance.get_schedule_ticks(simple.get_origin_id())
            assert ticks[0].status == ScheduleTickStatus.SUCCESS


def test_bad_env_fn():
    with seven.TemporaryDirectory() as temp_dir:
        with environ({'DAGSTER_HOME': temp_dir}):
            instance = DagsterInstance.get()

            recon_repo = ReconstructableRepository.for_file(__file__, 'the_repo')
            bad_env_fn = recon_repo.get_reconstructable_schedule('bad_env_fn_schedule')
            result = sync_launch_scheduled_execution(bad_env_fn.get_origin())

            assert isinstance(result, ScheduledExecutionFailed)
            assert (
                'Error occurred during the execution of environment_dict_fn for schedule bad_env_fn_schedule'
                in result.errors[0].to_string()
            )

            run = instance.get_run_by_id(result.run_id)
            assert run.is_failure

            ticks = instance.get_schedule_ticks(bad_env_fn.get_origin_id())
            assert ticks[0].status == ScheduleTickStatus.SUCCESS


def test_bad_should_execute():
    with seven.TemporaryDirectory() as temp_dir:
        with environ({'DAGSTER_HOME': temp_dir}):
            instance = DagsterInstance.get()

            recon_repo = ReconstructableRepository.for_file(__file__, 'the_repo')
            bad_should_execute = recon_repo.get_reconstructable_schedule(
                'bad_should_execute_schedule'
            )

            with pytest.raises(DagsterSubprocessError):
                sync_launch_scheduled_execution(bad_should_execute.get_origin())

            ticks = instance.get_schedule_ticks(bad_should_execute.get_origin_id())
            assert ticks[0].status == ScheduleTickStatus.FAILURE
            assert (
                'Error occurred during the execution of should_execute for schedule bad_should_execute_schedule'
                in ticks[0].error.message
            )


def test_skip():
    with seven.TemporaryDirectory() as temp_dir:
        with environ({'DAGSTER_HOME': temp_dir}):
            instance = DagsterInstance.get()

            recon_repo = ReconstructableRepository.for_file(__file__, 'the_repo')
            skip = recon_repo.get_reconstructable_schedule('skip_schedule')
            result = sync_launch_scheduled_execution(skip.get_origin())

            assert isinstance(result, ScheduledExecutionSkipped)

            ticks = instance.get_schedule_ticks(skip.get_origin_id())
            assert ticks[0].status == ScheduleTickStatus.SKIPPED



import pytest

from dagster import seven
from dagster.api.execute_run import sync_cli_api_execute_run
from dagster.core.events import DagsterEventType
from dagster.core.instance import DagsterInstance

from .utils import get_foo_pipeline_handle, legacy_get_foo_pipeline_handle


@pytest.mark.parametrize(
    "repo_handle", [get_foo_pipeline_handle(), legacy_get_foo_pipeline_handle()],
)
def test_execute_run_api(repo_handle):
    with seven.TemporaryDirectory() as temp_dir:
        instance = DagsterInstance.local_temp(temp_dir)
        events = [
            event
            for event in sync_cli_api_execute_run(
                instance=instance,
                pipeline_origin=repo_handle.get_origin(),
                pipeline_name='foo',
                environment_dict={},
                mode='default',
                solids_to_execute=None,
            )
        ]

    assert len(events) == 12
    assert events[1].event_type_value == DagsterEventType.PIPELINE_START.value
    assert events[-1].event_type_value == DagsterEventType.PIPELINE_SUCCESS.value



import sys

from dagster.api.list_repositories import sync_list_repositories
from dagster.cli.api import LoadableRepositorySymbol
from dagster.utils import file_relative_path


def test_sync_list_python_file():
    python_file = file_relative_path(__file__, 'api_tests_repo.py')
    loadable_repo_symbols = sync_list_repositories(
        sys.executable, python_file=python_file, module_name=None
    ).repository_symbols

    assert isinstance(loadable_repo_symbols, list)
    assert len(loadable_repo_symbols) == 1
    assert isinstance(loadable_repo_symbols[0], LoadableRepositorySymbol)

    symbol = loadable_repo_symbols[0]

    assert symbol.repository_name == 'bar_repo'
    assert symbol.attribute == 'bar_repo'


def test_sync_list_python_file_multi_repo():
    python_file = file_relative_path(__file__, 'multiple_repos.py')
    loadable_repo_symbols = sync_list_repositories(
        sys.executable, python_file=python_file, module_name=None
    ).repository_symbols

    assert isinstance(loadable_repo_symbols, list)
    assert len(loadable_repo_symbols) == 2
    assert isinstance(loadable_repo_symbols[0], LoadableRepositorySymbol)
    assert isinstance(loadable_repo_symbols[1], LoadableRepositorySymbol)

    by_symbol = {lrs.attribute: lrs for lrs in loadable_repo_symbols}

    assert by_symbol['repo_one_symbol'].repository_name == 'repo_one'
    assert by_symbol['repo_two'].repository_name == 'repo_two'


def test_sync_list_python_module():
    loadable_repo_symbols = sync_list_repositories(
        sys.executable, python_file=None, module_name='dagster.utils.test.hello_world_repository',
    ).repository_symbols

    assert isinstance(loadable_repo_symbols, list)
    assert len(loadable_repo_symbols) == 1
    assert isinstance(loadable_repo_symbols[0], LoadableRepositorySymbol)

    symbol = loadable_repo_symbols[0]

    assert symbol.repository_name == 'hello_world_repository'
    assert symbol.attribute == 'hello_world_repository'



# Normally we'd use dagster.seven for this, but we want to delay importing anything under the
# dagster package
try:
    from unittest import mock
except ImportError:
    # Because this dependency is not encoded setup.py deliberately
    # (we do not want to override or conflict with our users mocks)
    # we never fail when importing this.

    # This will only be used within *our* test environment of which
    # we have total control
    try:
        import mock
    except ImportError:
        pass


def test_no_warnings_on_import():
    with mock.patch('warnings.warn') as warn_mock:
        import dagster  # pylint: disable=unused-import

        expected_warnings = [
            'SOCKS support in urllib3 requires the installation of optional dependencies'
        ]
        for call in warn_mock.call_args_list:
            assert any(expected_warning in str(call) for expected_warning in expected_warnings)
